#!/usr/bin/perl -w -I/usr/sausalito/perl -I/usr/sausalito/handlers/base/vsite
# Copyright 2001 Sun Microsystems, Inc.  All rights reserved.
# $Id: network_destroy.pl,v 1.5 2001/12/04 07:28:44 pbaltz Exp $
#
# ensure that there is always an interface or alias with the IP address of
# each vsite

use CCE;
use Vsite;

my $cce = new CCE;
$cce->connectfd();

# get network info
my $network = $cce->event_object();
my $old_network = $cce->event_old();

# make sure there are no vsites using the ip address that is going away
# use the old ip address value, because that is always available
# and will be correct in case enabled and ipaddr are changing simultaneously
my @vsites = $cce->find('Vsite', { 'ipaddr' => $old_network->{ipaddr} });
my @networks = $cce->find('Network', 
			{ 
				'ipaddr' => $old_network->{ipaddr},
				'enabled' => 1
			 });

if (!$network->{enabled} && scalar(@vsites) && !scalar(@networks))
{
	# just fail if it is an alias
	if (!$network->{real})
	{
		&fail($cce);
	}
	else
	{
		# interface is going down, try to migrate to alias on another real
		# interface
		my $candidate = &get_candidate_iface($cce);
	
		if (!$candidate) { &fail($cce); }
		vsite_add_network_interface($cce, $old_network->{ipaddr}, $candidate);
	}
		
}
elsif ($old_network->{ipaddr} && scalar(@vsites) &&
        $network->{ipaddr} ne $old_network->{ipaddr})
{
    # make sure there is an interface for this ip after the change
    $network->{device} =~ /^([^:]+)/;
    vsite_add_network_interface($cce, $old_network->{ipaddr}, $1);
}

$cce->bye('SUCCESS');
exit(0);

sub get_candidate_iface
{
	# keep it simple for now
	return 'eth0';
}

sub fail
{
	my $cce = shift;

    $cce->bye('FAIL', '[[base-vsite.vsiteUsingIpAddress]]');
    exit(1);
}
