#!/usr/bin/perl -I/usr/sausalito/perl
# Copyright 2001 Sun Microsystems, Inc.  All rights reserved.
# $Id: update_site_admins.pl,v 1.6 2001/12/07 00:27:13 pbaltz Exp $
#
# keep all site admins capabilities in sync as admin users give and
# take away features from a virtual site

use CCE;
use Base::User::Capabilities;

my $cce = new CCE;
$cce->connectfd();

my $vsite = $cce->event_object();
my $vsite_old = {};

my @site_admins = ();

# handle site admin creation and modifying the siteAdminCaps property
if ($vsite->{CLASS} eq 'Vsite')
{
	$vsite_old = $cce->event_old();
	@site_admins = $cce->findx('User', 
					{ 'site' => $vsite->{name}, 'capLevels' => 'siteAdmin' },
					{}, '', '');
}
else
{
	# must be a user
	if ($vsite->{capLevels} !~ /siteAdmin/)
	{
		$cce->bye('SUCCESS');
		exit(0);
	}

	push @site_admins, $cce->event_oid();
	
	# need to get the vsite info if this is a user
	my ($oid) = $cce->findx('Vsite', { 'name' => $vsite->{site} });
	(my $ok, $vsite, $vsite_old) = $cce->get($oid);

	if (!$ok)
	{
		$cce->bye('FAIL', '[[base-vsite.cantReadVsite]]');
		exit(1);
	}
}

# setup lists of caps for user updates
my @old_caps = $cce->scalar_to_array($vsite_old->{siteAdminCaps});
my @new_caps = $cce->scalar_to_array($vsite->{siteAdminCaps});

# get the caps class
my $capabilities = new Base::User::Capabilities($cce);

# go through and reset all site admins' capLevels
for my $admin (@site_admins)
{
	if (!$capabilities->modUserCaps($admin, \@old_caps, \@new_caps))
	{
		$cce->bye('FAIL', '[[base-vsite.cantUpdateSiteAdminCaps]]');
		exit(0);
	}
}

$cce->bye('SUCCESS');
exit(0);
